@@ -25,11 +25,6 @@ char *gistPathDefault= GISTPATH;
 
 #include <string.h>
 
-extern double strtod(const char *, char **);
-extern long strtol(const char *, char **, int);
-
-extern char *getenv(const char *);
-
 struct GsysRead {
   char *legend;
   GpBox viewport;
@@ -83,30 +78,50 @@ static char *LegendsRead(char *input, struct GlegRead *dest);
    not done.  '~' is NOT recognized in the given filename.
  */
 
-static char *scratch;
+extern char *g_argv0;
+char *g_argv0 = 0;
+
+static char *scratch = 0;
+static char *gist_path = 0;
 
 static char *FormGistPath(void)
 {
-  char *gistPath= getenv("GISTPATH");
-  int len= gistPath? strlen(gistPath) : 0;
-  char *place;
+  if (!gist_path) {
+    char *gistPath = getenv("GISTPATH");
+    int len = gistPath? strlen(gistPath) : 0;
+    int len0 = g_argv0? strlen(g_argv0) : 0;
+    int lend = gistPathDefault? strlen(gistPathDefault) : 0;
+    char *place;
 
-  /* Get enough scratch space to hold 1023 characters of current path name,
-     plus the concatenation of the GISTPATH environment variable and the
-     GISTPATH compile-time option */
-  scratch= p_malloc(len+strlen(gistPathDefault)+1026);
-  if (!scratch) return 0;
+    /* Get enough scratch space to hold
+       the concatenation of the GISTPATH environment variable and the
+       GISTPATH compile-time option, and a fallback computed from argv[0] */
+    gist_path = p_malloc(len+len0+lend+4);
+    if (!gist_path) return 0;
 
-  place= scratch+1024;
-  if (gistPath) {
-    strcpy(place, gistPath);
-    place+= len;
-    *place= ':';
-    place++;
+    place = gist_path;
+    if (gistPath) {
+      strcpy(place, gistPath);
+      place += len;
+      *(place++) = ':';
+    }
+    strcpy(place, gistPathDefault);
+    place += lend;
+    /* back up to sibling of directory containing executable */
+    for (len=len0-1 ; len>0 ; len--) if (g_argv0[len]=='/') break;
+    for (len-- ; len>0 ; len--) if (g_argv0[len]=='/') break;
+    if (len) {
+      /* tack /g/ sibling of executable directory onto path */
+      *(place++) = ':';
+      strncpy(place, g_argv0, ++len);
+      place += len;
+      strcpy(place, "g");
+    }
   }
-  strcpy(place, gistPathDefault);
 
-  return scratch+1024;
+  scratch = p_malloc(1028);
+  if (!scratch) return 0;
+  return gist_path;
 }
 
 static char *FormFullName(char *gistPath, const char *name)
@@ -119,6 +134,11 @@ static char *FormFullName(char *gistPath, const char *name)
     /* Skip past any components of the GISTPATH which result in impossibly
        long path names */
     do len= strcspn(gistPath, ":"); while (!len);
+    /* handle MS Windows drive letters */
+    if (len==1 && gistPath[1]==':' &&
+        ((gistPath[0]>='A' && gistPath[0]<='Z') ||
+         (gistPath[0]>='a' && gistPath[0]<='z')))
+      len = 2+strcspn(gistPath+2, ":");
     if (!len) break;
     elen= len;
 
@@ -309,9 +329,9 @@ int GpReadPalette(Engine *engine, const char *gpFile,
       if (nextCell>=nColors) nextCell= oldCell;
       frac= old-(double)oldCell;
       frac1= 1.0-frac;
-      r = frac1*P_R(pal[oldCell]) + frac*P_R(pal[nextCell]);
-      g = frac1*P_G(pal[oldCell]) + frac*P_G(pal[nextCell]);
-      b = frac1*P_B(pal[oldCell]) + frac*P_B(pal[nextCell]);
+      r = (int)(frac1*P_R(pal[oldCell]) + frac*P_R(pal[nextCell]));
+      g = (int)(frac1*P_G(pal[oldCell]) + frac*P_G(pal[nextCell]));
+      b = (int)(frac1*P_B(pal[oldCell]) + frac*P_B(pal[nextCell]));
       pal[newCell] = P_RGB(r, g, b);
       /*if (!lookingFor4)
         pal[newCell].gray= frac1*pal[oldCell].gray+frac*pal[nextCell].gray;*/
