@@ -30,6 +30,7 @@ extern double log10(double);
 static void HandleExpose(Engine *engine, Drauing *drawing, int *xy);
 static void HandleClick(Engine *e,int b,int md,int x,int y, unsigned long ms);
 static void HandleMotion(Engine *e,int md,int x,int y);
+static void HandleKey(Engine *e,int k,int md);
 
 static void RedrawMessage(FXEngine *fxe);
 static void MovePointer(FXEngine *fxe, Drauing *drawing,
@@ -131,13 +132,13 @@ GpFXEngine(char *name, int landscape, int dpi, char *displayName)
   fxe->buttonState = 0;
   fxe->iSystem = -1;
   strcpy(fxe->msgText, "Press 1, 2, 3 to zoom in, pan, zoom out");
-  fxe->msgWidth = p_txwidth(s, "Press 1, 2, 3 to zoom in, pan, zoom out",
-                            39, P_GUI_FONT, 15);
+  fxe->msglen = 0;
   fxe->zoomState = fxe->zoomSystem = fxe->zoomAxis = 0;
   fxe->zoomX = fxe->zoomY = 0.0;
 
   /* set the fancy event handlers */
-  GxInput((Engine *)fxe, &HandleExpose, &HandleClick, &HandleMotion, 0);
+  GxInput((Engine *)fxe, &HandleExpose, &HandleClick, &HandleMotion,
+          &HandleKey);
 
   return (Engine *)fxe;
 }
@@ -218,6 +219,59 @@ xf_region(FXEngine *fxe, int x, int y)
   return 0;
 }
 
+extern void (*g_on_keyline)(char *msg);
+void (*g_on_keyline)(char *msg) = 0;
+
+static void
+HandleKey(Engine *e,int k,int md)
+{
+  FXEngine *fxe = (FXEngine *)e;
+  if (!fxe->pressed && !fxe->buttonState) {
+    if (!fxe->msglen) fxe->msgText[0] = '\0';
+    if (k>=' ' && k<'\177') {
+      /* append printing characters */
+      if (fxe->msglen>=94) fxe->msglen = 0;  /* crude overflow handling */
+      fxe->msgText[fxe->msglen++] = k;
+      fxe->msgText[fxe->msglen] = '\0';
+    } else if (k=='\177' || k=='\010') {
+      /* delete or backspace kills char */
+      if (fxe->msglen)
+        fxe->msgText[--fxe->msglen] = '\0';
+    } else if (k=='\027') {
+      /* C-w kills word */
+      int n = fxe->msglen;
+      char c = n? fxe->msgText[n-1] : '\0';
+      if (c<'0' || (c>'9' && c<'A') || (c>'Z' && c<'a' && c!='_') || c>'z') {
+        if (fxe->msglen)
+          fxe->msgText[--fxe->msglen] = '\0';
+      } else {
+        while (--n) {
+          c = fxe->msgText[n-1];
+          if (c<'0' || (c>'9' && c<'A') || (c>'Z' && c<'a' && c!='_') ||
+              c>'z') break;
+        }
+        fxe->msgText[n] = '\0';
+        fxe->msglen = n;
+      }
+    } else if (k=='\025') {
+      /* C-u kills line */
+      fxe->msgText[0] = '\0';
+      fxe->msglen = 0;
+    } else if (k=='\012' || k=='\015') {
+      /* linefeed or carriage return sends line to interpreter */
+      int n = fxe->msglen;
+      fxe->msgText[n] = '\012';
+      fxe->msgText[n+1] = '\0';
+      p_stdout(fxe->msgText);
+      fxe->msgText[n] = '\0';
+      if (g_on_keyline) g_on_keyline(fxe->msgText);
+      fxe->msgText[0] = '\0';
+      fxe->msglen = 0;
+    }
+    RedrawMessage(fxe);
+  }
+}
+
 /* ------------------------------------------------------------------------ */
 
 static void
@@ -415,22 +469,24 @@ MovePointer(FXEngine *fxe, Drauing *drawing,
     locked = 0;
   }
 
-  if (system) {
-    logX = system->flags&D_LOGX;
-    logY = system->flags&D_LOGY;
-    xWC = GetFormat(xFormat, xWC, system->trans.window.xmin,
-                    system->trans.window.xmax, logX);
-    f1 = xFormat;
-    yWC = GetFormat(yFormat, yWC, system->trans.window.ymin,
-                    system->trans.window.ymax, logY);
-    f2 = yFormat;
-  } else {
-    f1 = f2 = stdFormat;
-  }
-  sprintf(format, "%%s%%2d (%s, %s)", f1, f2);
-  sprintf(fxe->msgText, format, locked? "=" : ":", iSystem, xWC, yWC);
+  if (!fxe->msglen) {
+    if (system) {
+      logX = system->flags&D_LOGX;
+      logY = system->flags&D_LOGY;
+      xWC = GetFormat(xFormat, xWC, system->trans.window.xmin,
+                      system->trans.window.xmax, logX);
+      f1 = xFormat;
+      yWC = GetFormat(yFormat, yWC, system->trans.window.ymin,
+                      system->trans.window.ymax, logY);
+      f2 = yFormat;
+    } else {
+      f1 = f2 = stdFormat;
+    }
+    sprintf(format, "%%s%%2d (%s, %s)", f1, f2);
+    sprintf(fxe->msgText, format, locked? "=" : ":", iSystem, xWC, yWC);
 
-  RedrawMessage(fxe);
+    RedrawMessage(fxe);
+  }
   if (rubberBanding) DrawRubber(fxe, x, y);
 }
 
@@ -692,7 +748,8 @@ static int FindSystem(FXEngine *fxe, Drauing *drawing, int x, int y,
     min= 9.; /* now assume mouse is closer to an axis than 9 units */
     for (i=nSystems ; i>0 ; i--) {
       sys= (GeSystem *)sys->el.prev;
-      box= &sys->trans.viewport; 
+      if (!sys->elements) continue;
+      box= &sys->trans.viewport;
       if (yn>=box->ymin && yn<=box->ymax) {
 	tmp= xn-box->xmax;
 	if(tmp<min && tmp>0) { min= tmp; iSystem= i; thesys= sys; }
