@@ -28,6 +28,7 @@ extern char **ym_argv;
 extern void (*CleanUpForExit)(void);
 static void (*moreCleanUp)(void)= 0;
 static void mpy_CleanUp(void);
+extern void ym_escape(void);
 
 /* mpy_from is the communication between Y_mp_recv and Y_mp_from */
 static int mpy_from= -1;
@@ -218,6 +219,10 @@ static void mpy_CleanUp(void)
 {
   if (moreCleanUp) moreCleanUp();
   if (mpy_size) {
+#ifdef DEBUG
+    printf("(%d) doing mpy_CleanUp\n", (int)mpy_rank);
+    fflush(stdout);
+#endif
     mpy_iquit= 1;
     if (mpy_f_msg) mpy_fatal(mpy_f_origin, &mpy_f_msg[1]);
     mpy_quit();
@@ -778,7 +783,7 @@ void Y_mpy_sync(int nArgs)
 #endif
       if (mpy_control_msg[0]!=MPY_BEGIN) {
 	if (mpy_control_msg[0]==MPY_QUIT) mpy_quit();
-	YError("mpy_sync expecting BEGIN message from rank 0 mpy_sync");
+	YError("mpy_sync expecting BEGIN message from rank 0 mpy_sync!!");
       }
       mpy_begin++;
     }
@@ -1325,10 +1330,10 @@ static void mpy_fatal(int origin, char *msg)
     mpy_f_msg= mess;
     if (!mpy_iquit) {
 #ifdef DEBUG
-      printf("(%d) mpy_fatal calling Y_quit(0)\n", (int)mpy_rank);
+      printf("(%d) mpy_fatal calling ym_escape\n", (int)mpy_rank);
       fflush(stdout);
 #endif
-      Y_quit(0);
+      ym_escape();
     }
   }
 
@@ -1367,11 +1372,11 @@ static void mpy_quit(void)
   if (!mpy_iquit) {
     extern void Y_quit(int nArgs);
 #ifdef DEBUG
-    printf("(%d) mpy_quit calling Y_quit(0)\n", (int)mpy_rank);
+    printf("(%d) mpy_quit calling ym_escape()\n", (int)mpy_rank);
     fflush(stdout);
 #endif
     if (!mpy_rank) YWarning("MPY_QUIT received on rank 0?");
-    Y_quit(0);
+    ym_escape();
   }
 
   if (mpy_rank) {
