@@ -12,12 +12,9 @@ void
 p_fill(p_win *w, int convexity)
 {
   int n = w_pt_count;
-  HDC dc = w_getdc(w, 4);
+  HDC dc = w_getdc(w, 12);
   if (dc) {
     /* SetPolyFillMode(dc, ALTERNATE); */
-    BeginPath(dc);
     Polygon(dc, w_pt_list, n);
-    EndPath(dc);
-    FillPath(dc);
   }
 }
