@@ -14,6 +14,8 @@
 #include "pstdlib.h"
 #include <string.h>
 
+#include <errno.h>
+
 /*--------------------------------------------------------------------------*/
 
 extern VMaction Power, Multiply, Divide, Modulo, Add, Subtract,
@@ -559,7 +561,7 @@ void opname(Operand *l, Operand *r) \
   if (dst) { \
     long i, n= l->type.number; \
     typd *lv= l->value, *rv= r->value; \
-    for (i=0 ; i<n ; i++) dst[i]= RMOD(lv[i],rv[i]); \
+    for (i=0 ; i<n ; i++) dst[i]= (typd)RMOD(lv[i],rv[i]); \
     Popper(l->owner); \
   } else ModuloError(); }
 
@@ -586,8 +588,16 @@ void PowerII(void) { Symbol *spr= sp--;
   sp->value.i= powLL(sp->value.i, spr->value.i); }
 void PowerIL(void) { Symbol *spr= sp--;
   sp->value.l= powLL(sp->value.i, spr->value.l); sp->ops= &longScalar; }
-void PowerID(void) { Symbol *spr= sp--;
-  sp->value.d= pow(sp->value.i, spr->value.d); sp->ops= &doubleScalar; }
+void PowerID(void) {
+  Symbol *spr= sp--;
+  errno = 0;
+  sp->value.d= pow(sp->value.i, spr->value.d);
+  if (errno) {
+    if (errno!=ERANGE || sp->value.d!=0.)
+      YError("mathlib pow() function signals error");
+  }
+  sp->ops= &doubleScalar;
+}
 void PowerIB(void)
 { Power_BB(FormOperandIS(sp-1, &lop), FormOperandDB(sp, &rop)); }
 
@@ -650,7 +660,7 @@ void opname(Operand *l, Operand *r) \
   if (dst) { \
     long i, n= l->type.number; \
     typd *lv= l->value; long *rv= r->value; \
-    for (i=0 ; i<n ; i++) dst[i]= powLL(lv[i], rv[i]); \
+    for (i=0 ; i<n ; i++) dst[i]= (typd)powLL(lv[i], rv[i]); \
     Popper(l->owner); \
   } else PowerError(); }
 
@@ -666,7 +676,7 @@ void opname(Operand *l, Operand *r) \
   if (dst) { \
     long i, n= l->type.number; \
     typd *lv= l->value; long *rv= r->value; \
-    for (i=0 ; i<n ; i++) dst[i]= powDL(lv[i], rv[i]); \
+    for (i=0 ; i<n ; i++) dst[i]= (typd)powDL(lv[i], rv[i]); \
     Popper(l->owner); \
   } else PowerError(); }
 
@@ -689,7 +699,9 @@ void opname(Operand *l, Operand *r) \
   if (dst) { \
     long i, n= l->type.number; \
     typd *lv= l->value, *rv= r->value; \
-    for (i=0 ; i<n ; i++) dst[i]= pow(lv[i], rv[i]); \
+    for (i=0,errno=0 ; i<n ; i++) { dst[i]= (typd)pow(lv[i], rv[i]); \
+    if (errno) { if (errno==ERANGE && !dst[i]) errno=0; \
+                 else YError("mathlib pow() function signals error"); }} \
     Popper(l->owner); \
   } else PowerError(); }
 
@@ -700,7 +712,8 @@ void PowerXZ(Operand *l, Operand *r)
   if (dst) {
     long i, n= 2*l->type.number;
     double *lv= l->value, *rv= r->value;
-    for (i=0 ; i<n ; i+=2) powZZ(&dst[i], &lv[i], &rv[i]);
+    for (i=0,errno=0 ; i<n && !errno ; i+=2) powZZ(&dst[i], &lv[i], &rv[i]);
+    if (errno) YError("mathlib error in complex^complex");
     PopTo(l->owner);
   } else PowerError();
 }
