@@ -17,9 +17,11 @@
 #include <string.h>
 #include <stdio.h>
 
+#include <errno.h>
+
 extern char *yVersion;
 #ifndef Y_VERSION
-#define Y_VERSION "<unknown>"
+#include "yversion.h"
 #endif
 char *yVersion= Y_VERSION;
 
@@ -68,6 +70,7 @@ extern int y_on_idle(void);
 extern void y_on_stdin(char *input_line);
 extern void y_on_exception(int signal, char *errmsg);
 extern int y_on_quit(void);
+extern void (*g_on_keyline)(char *);
 
 extern int yBatchMode;  /* may be set with -batch, see std0.c */
 
@@ -80,8 +83,10 @@ on_launch(int argc, char *argv[])
   ym_argv= argv;
   p_mmfail = y_mmfail;
   YInit(yBuiltIns, yGlobals, yLinkNames, initialIncludes);
-  if (!yBatchMode)
+  if (!yBatchMode) {
     p_stdinit(y_on_stdin);
+    g_on_keyline = y_on_stdin;
+  }
   p_idler(y_on_idle);
   p_handler(y_on_exception);
   p_quitter(y_on_quit);
@@ -248,6 +253,8 @@ void Y_set_site(int nArgs)
           p_fclose(test);
           path[i+1] = '\0';
           ySiteDir= path;
+          /* if ySiteDir was bad, so is yHomeDir, guess the same */
+          yHomeDir = p_strcpy(ySiteDir);  
         } else {
           p_free(path);
         }
@@ -902,6 +909,7 @@ static void UnaryTemplate(int nArgs, Looper *DLooper, Looper *ZLooper)
   if (nArgs != 1) YError("expecting exactly one argument");
   sp->ops->FormOperand(sp, &op);
   promoteID= op.ops->promoteID;
+  errno = 0;
   if (promoteID<=T_DOUBLE) {
     if (promoteID<T_DOUBLE) op.ops->ToDouble(&op);
     DLooper(BuildResultU(&op, &doubleStruct), op.value, op.type.number);
@@ -911,6 +919,7 @@ static void UnaryTemplate(int nArgs, Looper *DLooper, Looper *ZLooper)
     ZLooper(BuildResultU(&op, &complexStruct), op.value, 2*op.type.number);
     PopTo(sp-2);
   }
+  if (errno) YError("mathlib function signals error");
   Drop(1);
 }
 
@@ -962,7 +971,7 @@ static Looper sinLoop, sinZLoop;
 static void sinLoop(double *dst, double *src, long n)
 { long i; for (i=0 ; i<n ; i++) dst[i]= sin(src[i]); }
 static void sinZLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i+=2) sinZ(&dst[i], &src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i+=2) sinZ(&dst[i], &src[i]); }
 
 void Y_sin(int nArgs) { UnaryTemplate(nArgs, &sinLoop, &sinZLoop); }
 
@@ -973,7 +982,7 @@ static Looper cosLoop, cosZLoop;
 static void cosLoop(double *dst, double *src, long n)
 { long i; for (i=0 ; i<n ; i++) dst[i]= cos(src[i]); }
 static void cosZLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i+=2) cosZ(&dst[i], &src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i+=2) cosZ(&dst[i], &src[i]); }
 
 void Y_cos(int nArgs) { UnaryTemplate(nArgs, &cosLoop, &cosZLoop); }
 
@@ -984,7 +993,7 @@ static Looper tanLoop, tanZLoop;
 static void tanLoop(double *dst, double *src, long n)
 { long i; for (i=0 ; i<n ; i++) dst[i]= tan(src[i]); }
 static void tanZLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i+=2) tanZ(&dst[i], &src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i+=2) tanZ(&dst[i], &src[i]); }
 
 void Y_tan(int nArgs) { UnaryTemplate(nArgs, &tanLoop, &tanZLoop); }
 
@@ -993,9 +1002,9 @@ void Y_tan(int nArgs) { UnaryTemplate(nArgs, &tanLoop, &tanZLoop); }
 static Looper asinLoop, asinZLoop;
 
 static void asinLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i++) dst[i]= asin(src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i++) dst[i]= asin(src[i]); }
 static void asinZLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i+=2) asinZ(&dst[i], &src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i+=2) asinZ(&dst[i], &src[i]); }
 
 void Y_asin(int nArgs) { UnaryTemplate(nArgs, &asinLoop, &asinZLoop); }
 
@@ -1004,9 +1013,9 @@ void Y_asin(int nArgs) { UnaryTemplate(nArgs, &asinLoop, &asinZLoop); }
 static Looper acosLoop, acosZLoop;
 
 static void acosLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i++) dst[i]= acos(src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i++) dst[i]= acos(src[i]); }
 static void acosZLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i+=2) acosZ(&dst[i], &src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i+=2) acosZ(&dst[i], &src[i]); }
 
 void Y_acos(int nArgs) { UnaryTemplate(nArgs, &acosLoop, &acosZLoop); }
 
@@ -1017,7 +1026,7 @@ static Looper atanLoop, atanZLoop;
 static void atanLoop(double *dst, double *src, long n)
 { long i; for (i=0 ; i<n ; i++) dst[i]= atan(src[i]); }
 static void atanZLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i+=2) atanZ(&dst[i], &src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i+=2) atanZ(&dst[i], &src[i]); }
 
 void Y_atan(int nArgs)
 {
@@ -1038,7 +1047,9 @@ void Y_atan(int nArgs)
     if (!dst) YError("operands not conformable in binary atan");
     y= opy.value;
     x= opx.value;
-    for (i=0 ; i<opy.type.number ; i++) dst[i]= atan2(y[i], x[i]);
+    errno = 0;
+    for (i=0 ; i<opy.type.number && !errno ; i++) dst[i]= atan2(y[i], x[i]);
+    if (errno) YError("mathlib atan2() function signals error");
     PopToD(sp-3);
     Drop(2);
   }
@@ -1049,9 +1060,9 @@ void Y_atan(int nArgs)
 static Looper sinhLoop, sinhZLoop;
 
 static void sinhLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i++) dst[i]= sinh(src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i++) dst[i]= sinh(src[i]); }
 static void sinhZLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i+=2) sinhZ(&dst[i], &src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i+=2) sinhZ(&dst[i], &src[i]); }
 
 void Y_sinh(int nArgs) { UnaryTemplate(nArgs, &sinhLoop, &sinhZLoop); }
 
@@ -1060,9 +1071,9 @@ void Y_sinh(int nArgs) { UnaryTemplate(nArgs, &sinhLoop, &sinhZLoop); }
 static Looper coshLoop, coshZLoop;
 
 static void coshLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i++) dst[i]= cosh(src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i++) dst[i]= cosh(src[i]); }
 static void coshZLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i+=2) coshZ(&dst[i], &src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i+=2) coshZ(&dst[i], &src[i]); }
 
 void Y_cosh(int nArgs) { UnaryTemplate(nArgs, &coshLoop, &coshZLoop); }
 
@@ -1073,7 +1084,7 @@ static Looper tanhLoop, tanhZLoop;
 static void tanhLoop(double *dst, double *src, long n)
 { long i; for (i=0 ; i<n ; i++) dst[i]= tanh(src[i]); }
 static void tanhZLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i+=2) tanhZ(&dst[i], &src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i+=2) tanhZ(&dst[i], &src[i]); }
 
 void Y_tanh(int nArgs) { UnaryTemplate(nArgs, &tanhLoop, &tanhZLoop); }
 
@@ -1082,9 +1093,15 @@ void Y_tanh(int nArgs) { UnaryTemplate(nArgs, &tanhLoop, &tanhZLoop); }
 static Looper expLoop, expZLoop;
 
 static void expLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i++) dst[i]= exp(src[i]); }
+{
+  long i;
+  for (i=0 ; i<n ; i++) {
+    dst[i]= exp(src[i]);
+    if (errno) { if (errno==ERANGE && !dst[i]) errno=0; else break; }
+  }
+}
 static void expZLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i+=2) expZ(&dst[i], &src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i+=2) expZ(&dst[i], &src[i]); }
 
 void Y_exp(int nArgs) { UnaryTemplate(nArgs, &expLoop, &expZLoop); }
 
@@ -1093,9 +1110,9 @@ void Y_exp(int nArgs) { UnaryTemplate(nArgs, &expLoop, &expZLoop); }
 static Looper logLoop, logZLoop;
 
 static void logLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i++) dst[i]= log(src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i++) dst[i]= log(src[i]); }
 static void logZLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i+=2) logZ(&dst[i], &src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i+=2) logZ(&dst[i], &src[i]); }
 
 void Y_log(int nArgs) { UnaryTemplate(nArgs, &logLoop, &logZLoop); }
 
@@ -1104,7 +1121,7 @@ void Y_log(int nArgs) { UnaryTemplate(nArgs, &logLoop, &logZLoop); }
 static Looper log10Loop, log10ZLoop;
 
 static void log10Loop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i++) dst[i]= log10(src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i++) dst[i]= log10(src[i]); }
 static void log10ZLoop(double *dst, double *src, long n)
 { YError("log10(complex) not implemented, use log"); }
 
@@ -1115,7 +1132,7 @@ void Y_log10(int nArgs) { UnaryTemplate(nArgs, &log10Loop, &log10ZLoop); }
 static Looper sqrtLoop, sqrtZLoop;
 
 static void sqrtLoop(double *dst, double *src, long n)
-{ long i; for (i=0 ; i<n ; i++) dst[i]= sqrt(src[i]); }
+{ long i; for (i=0 ; i<n && !errno ; i++) dst[i]= sqrt(src[i]); }
 static void sqrtZLoop(double *dst, double *src, long n)
 { long i; for (i=0 ; i<n ; i+=2) sqrtZ(&dst[i], &src[i]); }
 
@@ -1240,7 +1257,7 @@ static void signLLoop(void *d, void *s, long n)
   for (i=0 ; i<n ; i++) dst[i]= src[i]<0? -1 : 1; }
 static void signFLoop(void *d, void *s, long n)
 { long i; float *dst= d; float *src= s;
-  for (i=0 ; i<n ; i++) dst[i]= src[i]<0? -1 : 1; }
+  for (i=0 ; i<n ; i++) dst[i]= src[i]<0? -1.f : 1.f; }
 static void signDLoop(void *d, void *s, long n)
 { long i; double *dst= d; double *src= s;
   for (i=0 ; i<n ; i++) dst[i]= src[i]<0? -1 : 1; }
