@@ -15,7 +15,7 @@
 #include <string.h>
 
 #ifndef PATH_SEP_DELIMIT
-#define PATH_SEP_DELIMIT ": \t"
+#define PATH_SEP_DELIMIT ":;"
 #endif
 
 extern void YError(const char *msg);
@@ -68,11 +68,48 @@ static long FindSource(long index);
 extern long ReopenSource(long index, int notExtern);
 
 static int yp_chk_hash(p_file *file, int context, char *line);
+static char *yp_strtok_text = 0;
+static char *yp_strtok(char *text, char *delim);
+static char *
+yp_strtok(char *text, char *delim)
+{
+  char *start = 0;
+  int i;
+  if (text) yp_strtok_text = text;
+  if (!yp_strtok_text) return 0;
+  text = yp_strtok_text;
+  /* skip leading delimiters */
+  while (text[0]) {
+    for (i=0 ; delim[i] ; i++) if (text[0]==delim[i]) break;
+    if (!delim[i]) break;
+    text++;
+  }
+  if (!text[0]) {
+    yp_strtok_text = 0;
+    return 0;
+  }
+  start = text;
+  /* the whole point is to skip Windows drive letter */
+  if (text[1]==':') text += 2;
+  /* scan until delimiter */
+  while (text[0]) {
+    for (i=0 ; delim[i] ; i++) if (text[0]==delim[i]) break;
+    if (delim[i]) break;
+    text++;
+  }
+  if (!text[0]) {
+    yp_strtok_text = 0;
+  } else {
+    text[0] = '\0';
+    yp_strtok_text = text+1;
+  }
+  return start;
+}
 
 void YpSetPaths(const char *pathlist)
 {
   char *paths= p_strcpy(pathlist);
-  char *token= strtok(paths, PATH_SEP_DELIMIT);
+  char *token= yp_strtok(paths, PATH_SEP_DELIMIT);
   char *prefix;
 
   ClearPrefixes();
@@ -82,7 +119,7 @@ void YpSetPaths(const char *pathlist)
     if (YIsDotRelative(token)) prefix= p_strcpy(token);
     else prefix= YExpandName(token);
     AddPrefix(prefix);
-    token= strtok((char *)0, PATH_SEP_DELIMIT);
+    token= yp_strtok((char *)0, PATH_SEP_DELIMIT);
   }
 
   p_free(paths);
